/*m547revc - 24 parm model revised*/
/* interact transition and ret effects, NO elig effects*/
/* interact state dep effects */


new 200,50000;
format /rd 8,4;
outwidth 250;
output file = m547revc.out reset;


library  maxlik;
#include maxlik.ext;
maxset;

nobs = 5617;


/*  Load in SSP data: cols 1-38=ia 39=expt2 40=ten 41-46=some xvars */
/*  NOTE in2.dat uses new eligibility date ten */

load temp[] = in54.dat;
ssp = reshape(temp,nobs,79);
clear temp;

mns=meanc(ssp);

vnames = { "ym1", "y0",
           "y1", "y2", "y3", "y4", "y5", "y6", "y7", "y8", "y9", "y10",
           "y11", "y12", "y13", "y14", "y15", "y16", "y17", "y18", "y19", "y20",
           "y21", "y22", "y23", "y24", "y25", "y26", "y27", "y28", "y29", "y30",
           "y31", "y32", "y33", "y34", "y35", "y36", "y37", "y38", "y39", "y40",
           "y41", "y42", "y43", "y44", "y45", "y46", "y47", "y48", "y49", "y50",
           "y51", "y52", "y53", "y54", "y55", "y56", "y57", "y58", "y59", "y60",
           "y61", "y62", "y63", "y64", "y65", "y66", "y67", "y68", "y69",
           "expt", "ten", "hsgrad", "basework", "exper", "noexp",
           "agelt25", "premos" }; 
print "means of input data set";
print mns;



 /*  r1= no. pts   aj=vector of pts   wj=wts
     abramowitz+stegun p. 924  */

r1=10;
aj = {-3.436159118837738,-2.532731674232790,-1.756683649299882,-1.036610829789514,-0.342901327223705,
       0.342901327223705,1.036610829789514,1.756683649299882,2.532731674232790,3.436159118837738 };

wj = {7.640432855233e-6,1.343645764781e-3,3.387439445548e-2,2.401386110823e-1,6.108626337353e-1,
      6.108626337353e-1,2.401386110823e-1,3.387439445548e-2,1.343645764781e-3,7.640432855233e-6};

print "gaussian quadrature (hermite integration) number pts=" r1;
gq=aj~wj;
print "pts and weights from abramowitz & stegun p. 924";
print gq;


	/*  Define variable names in LFP dataset for logit models  */

nobs=rows(ssp);

saved(ssp,"ssp1",vnames);


__maxvec = 600000;


 /* estimation */

_max_CovPar = 2; /*  1=Inv. Hessian; 2=XProduct of Scores; 3=QML */
_mlgrdh = 5e-4;
_mlstep = 5;  /*  Max gradient step */
_max_GradTol = 1e-4; /*  Grad tolerance: default = 1e-5  */
_max_Algorithm=5;

print "covariance matrix choice=" _max_CovPar;
print "1=inv H 2=outer prod 3=qml" ;
print "algorithm=" _max_Algorithm;
print "2=bfgs 3=dfp 5=bhhh";
print "___" ;
print "basic selection mode (5-18 rev) quartic trend";
print "no elig effects, trans+ret effs interacted";
print "state dependence effects interacted";

clear b0;

b0 = {-1.2656, -1.0152, 4.1502, -8.3353, 5.588,
      4.7876 , 1.9262, -0.9103, 1.1198,
       -2.2449, .1943, .5556, .2137,
      -.5467, -.1141, -3.239 , -1.7082,
      -1.0697, -.5453, .0343,  -.1899,
       -.7772, -.3183, .7681 };

print "starting values" b0;


{bex,lex,gex,covex,retex} =
   maxprt( maxlik("ssp1",vnames,&likexog,b0) );
lex = nobs*lex;

print "Maximized log-likelihood= " lex;

/* su=exp(bex[8]/2); */
/* print "sigma of the random effect= " su; */


/*likelihood*/


proc likexog(theta,dat);
local nrows, fai, j, s2u, g, g2, g12,
      yt, ldv, l2dv, l12dv, re, expt, treat,
      t, tr, tsq, tcu, tqu, trend, arg1, ten, dt, pe,
      v1 , m;


nrows = rows(dat);
s2u=exp(theta[9]);
expt=dat[.,72];

v1=ones(nrows,1);


/* adjust ten by reducing by 1 => 1 mo lag in IA behavior*/
ten=dat[.,73] -v1.*1;

fai = ones(nrows,r1)/sqrt(pi);

j=1;
do while j<=r1;
 re=sqrt(2*s2u)*aj[j];

g=theta[6]+theta[22]*re;
g2=theta[7]+theta[23]*re;
g12=theta[8]+theta[24]*re;
 t=3;
 do while t<=71;
   m=t-2;
   tr=m/10;
   tsq=m*m/1000;
   tcu=m*m*m/100000;
   tqu=m*m*m*m/10000000;
   trend=theta[1]+theta[2]*tr+theta[3]*tsq+theta[4]*tcu+theta[5]*tqu;

   yt=dat[.,t];
   ldv=dat[.,t-1];
   l2dv=dat[.,t-2];
   l12dv=ldv.*l2dv;

  /*treat effects are zero in month 1 */
  /*for ineligs get elig effects months 2-15*/
  /*for eligs get elig effects months 2 - (ten-1)*/
  /*          transitions effs months ten, ten+1, ten+2 */
  /*          ret effs         months ten+3...ten+35    */


   treat =
         (expt.==2 ) .* ( m.>=ten) .* (m.<=(ten+2) ).*
        (  (theta[16]+theta[14]*re).*(ldv.==1)
         + (theta[17]+theta[15]*re).*(ldv.==0)  )

      +  (expt.==2) .* ( m.>=(ten+3)) .*(m.<=(ten+35) ) .*
        (  ( theta[18]+theta[20]*re).*(ldv.==1)
         + ( theta[19]+theta[21]*re).*(ldv.==0)  ) ;

   /* selection argument with 1/m term */
   /* note fix for m=1 to prevent divide by 0 */
   /* pe = 1 for 1st month, earliest move is m=2 */

   dt= theta[10]+theta[11]*(m-1)/10 
       + theta[12]/( (m==1)*1 + (m-1) );

   arg1=v1.*(dt- theta[13]*re);

   pe= (expt.<=0).*v1
    +(expt.>0).*(m==1).*v1
    +(expt.==1).*( (m>=2).*(m<=15).*cdfnc(arg1) + (m>15).*v1  )
    +(expt.==2).*( (m>=2).*(m.<ten).*cdfnc(arg1)
                 + (m.==ten).*cdfn(arg1) + (m.>ten).*v1  );

   fai[.,j] = fai[.,j] .*pe
          .*(exp(yt.*(ldv*g+l2dv*g2+g12*l12dv+treat+v1.*(re+trend))))
          ./(1+exp(ldv*g+l2dv*g2+g12*l12dv+treat+v1.*(re+trend)));
   t=t+1;
  endo;


j=j+1;
endo;

retp( ln(fai*wj) );    /*  Return the log-likelihood of the sample  */
endp;



output off;
end;

